'''
Copyright (c) <2015> <Victor de Lorenzo's lab, http://www.cnb.csic.es/~meml/>
                     
Author: Angel Goni-Moreno, www.angelgm.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'''

from packages import *
from pixelAnalysis import *
from plotContours import *
from calculateOutline import *
from getCoords import *
from figure import *
from report import *

def iOneImage(self):
    writeReport("Beginning analysis. Please wait... \n",self)
    Analysis=pixelAnalysis(self.singleFile,self.colourSingle.get(),self)
    fig = createFigure('Pixel view (relative 0..1)')
    plotPixelView(Analysis,fig,self.colourSingle.get(),self)
    fig = createFigure('3D Mesh view')
    plot3DView(Analysis,fig,self.colourSingle.get(),self)
    fig = createFigure('Contour view')
    plotContourView(Analysis,fig,self.colourSingle.get(),self)
    Analysis=pixelAnalysis(self.singleFile,self.colourSingle.get(),self,"absolute")
    fig = createFigure('Pixel view (absolute 0..255)')
    plotPixelView(Analysis,fig,self.colourSingle.get(),self)
    writeReport("Done! \n",self)

def iTwoRegions(self):
    writeReport("Beginning analysis. Please wait... \n",self)
    Analysis1 = pixelAnalysis(self.region1File,self.colourRegion1.get(),self)
    Analysis2 = pixelAnalysis(self.region2File,self.colourRegion2.get(),self)
    fig = createFigure('Region1 pixel view')
    plotPixelView(Analysis1,fig,self.colourRegion1.get(),self)
    fig = createFigure('Region1 contour field')
    plotContourView(Analysis1,fig,self.colourRegion1.get(),self)
    fig = createFigure('Region2 pixel view')
    plotPixelView(Analysis2,fig,self.colourRegion2.get(),self)
    fig = createFigure('Region2 contour field')
    plotContourView(Analysis2,fig,self.colourRegion2.get(),self)
    fig = createFigure('Combined contours')
    plotMatchContours(Analysis1, Analysis2,fig,self.colourRegion1.get(),self.colourRegion2.get(),self)
    writeReport("Done! \n",self)
   
def iCytometryEmulator(self):
    writeReport("Beginning analysis. Please wait... \n",self)
    cellAnalysis = pixelAnalysis(self.phaseFile,'phase',self )
    fluoAnalysis = pixelAnalysis(self.fluoFile, self.colourFluo.get(),self, "absolute")
    writeReport("   Level lines being calculated \n",self)
    fig = createFigure('Contour (level) Lines')
    cellContourLines = plotContour(cellAnalysis, int(self.contoursNucleoid.get()))
    writeReport("   Cell recognition: \n",self)
    emptyShapesCell = calculateOutline(cellContourLines, int(self.minNucleoid.get()), int(self.maxNucleoid.get()),self)
    fig = createFigure('Cell recognition')
    writeReport("   Preparing data for plotting cells \n",self)
    plotPhase(emptyShapesCell,fig,self)
    writeReport("   Preparing data for cytometry \n",self) 
    cell_values = calculateCytometry(emptyShapesCell,fluoAnalysis,self)
    fig = createFigure('Cytometry Emulator - log')
    plotCytometry(cell_values, fig, self, "log")
    fig = createFigure('Cytometry Emulator')
    plotCytometry(cell_values,fig,self)
    fig = createFigure('Intensity map')
    plotCytometryMap(cell_values,fig,self)
    writeReport("Done! \n",self)

def iAnalysis(self):
    # Main image analysis
    writeReport("Beginning analysis. Please wait... \n",self)
    nucleoidAnalysis = pixelAnalysis(self.nucleoidFile, self.colourNucleoid.get(),self)
    spotsAnalysis = pixelAnalysis(self.spotsFile, self.colourSpots.get(),self)

    # Plot contour lines of image analysis output
    fig = createFigure('Contour (level) Lines')
    writeReport("   Level lines being calculated \n",self)
    (nucleoidContourLines, spotsContourLines) = plotContours(nucleoidAnalysis, spotsAnalysis, int(self.contoursNucleoid.get()), int(self.contoursSpots.get()))

    # Shape recognition
    writeReport("   Nucleoid recognition (inner shapes): \n",self)
    emptyShapesNucleoid = calculateOutline(nucleoidContourLines, int(self.minNucleoid.get()), int(self.maxNucleoid.get()),self)
    writeReport("   Nucleoid recognition (outer shapes): \n",self)
    emptyShapesExtended = calculateOutline(nucleoidContourLines, int(self.minExtended.get()), int(self.maxExtended.get()),self)
    writeReport("   Spots recognition: \n",self)
    emptyShapesPoints = calculateOutline(spotsContourLines, int(self.minSpots.get()), int(self.maxSpots.get()),self)

    # Plot Nucleoid vs Extended (image background)
    fig = createFigure('Contour recognition')
    writeReport("   Preparing data for plotting nucleoid (1/2) \n",self)
    plotNucleoidLines(emptyShapesNucleoid,fig,self)
    writeReport("   Preparing data for plotting nucleoid (2/2) \n",self)
    plotExtendedLines(emptyShapesExtended,fig,self)

    # Plot Spots (image background)
    fig = createFigure('Spot recognition')
    writeReport("   Preparing data for plotting spots \n",self)
    plotPoints(emptyShapesPoints,fig,self)

    # Plot Spots vs Nucleoid (blank background)
    fig = createFigure('Spots in nucleoid regions')
    writeReport("   Nucleoid vs. Spots (1/3) \n",self)
    plotNucleoidLinesWhiteBack(emptyShapesNucleoid,fig,self)
    writeReport("   Nucleoid vs. Spots (2/3) \n",self)
    plotExtendedLinesWhiteBack(emptyShapesExtended,fig,self)
    writeReport("   Nucleoid vs. Spots (3/3) \n",self)
    plotPointsWhiteBack(emptyShapesPoints,fig,self)

    ## Plot Spot fluorescence data regarding positioning
    #fig = createFigure('Spot intensity in regions')
    writeReport("   Spot intensity stats \n",self)
    emptyShapesPoints = calculateOutlineWithLevels(spotsContourLines, int(self.minSpots.get()), int(self.maxSpots.get()),self)
    plotIntensityStats(emptyShapesPoints,emptyShapesNucleoid,emptyShapesExtended,self)


    writeReport("Done! \n",self)
def plotData():    
    plt.show()



